/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you want to add, delete, or rename functions or slots, use
** Qt Designer to update this file, preserving your code.
**
** You should not define a constructor or destructor in this file.
** Instead, write your code in functions called init() and destroy().
** These will automatically be called by the form's constructor and
** destructor.
*****************************************************************************/
#include <libdaisy.h>

using namespace std;

daisyplayer_t daisy;
daisy_position *bookmark = NULL;
extern QTranslator qt;
static void cb_text        (void *self, void *);
static void cb_audio_done  (void *self);
static void cb_audio_next  (void *self, unsigned long int);
static void cb_id          (void *self, void *);
static void cb_error       (void *self, enum daisy_status, const char *);
static void cb_progress    (void *self, long int);

/**
 * Initiates the mainwindow.
 */
void MainWindow::init ()
{
    	setCaption (tr ("DaisyPlayer - GUI edition"));
    	daisy = daisy_init (this, cb_audio_done, cb_audio_next, cb_text, cb_id, cb_error, cb_progress); 
	connect (this, SIGNAL(displayText (const QString &)), textBrowser, SLOT (setText (const QString &)));
	total_time_box->setReadOnly (true);
    	bookmarkAction->setEnabled (false);
    	loadBookmarkAction->setEnabled (false);
	nextChapterAction->setEnabled (false);
   	nextPassageAction->setEnabled (false);
    	pauseAction->setEnabled (false);
    	playAction->setEnabled (false);
    	prevChapterAction->setEnabled (false);
    	prevPassageAction->setEnabled (false);
   	stopAction->setEnabled (false);
  	font_combobox->insertItem("Bitstream Vera Sans", 0);
   	font_combobox->insertItem("Helvetica", 1);
   	font_combobox->insertItem("Luxi Mono", 2);
   	font_combobox->insertItem("Nimbus Roman No 9L", 3);
   	font_combobox->insertItem("Times New Roman", 4);
 }

/**
 * Opens the filedialog and to load a new daisybook.
 */
void MainWindow::fileOpen ()
{	const char *constSource;
	char * charSource, *title, *time, buffer[300];
	int ret = 0;
	
	QString filename = QFileDialog::getOpenFileName (
                                "/home", "Daisy (*.ncx *.html)", this,
                                "file open", tr  ("Choose a book to open"));
                                
	/* If the string isn't empty. */
	if ( ! filename.isEmpty () )
	{	/* Stop the playback and clears the textbrowser in case we are playing a book. */
		daisy_stop (daisy);	
	    	textBrowser->setText ("");
	    	/* Converts from QString to char* and loads the book. */
	    	constSource = filename.latin1 ();
	    	charSource = new (char[500]);
	    	strcpy (charSource, constSource);
	   	ret = daisy_load (daisy, charSource);
	   	/* If we succesfully opened the book. */
	   	if (ret == 1)
	   	{
	  		statusBar()->message (tr ("File succesfully opened"), 2000);
			/* Get the title if found and puts it in the titlebar. */
	   	 	title = daisy_get_info (daisy, DAISY_BOOKINFO_TITLETEXT);
	  	  	if(title != NULL)
	 	   	{	
	 	   		snprintf(buffer, sizeof (buffer), "DaisyPlayer - GUI edition -- %s", title);
	    			setCaption(buffer);
		    	}
		    	/* Get the totaltime of the book if found. */
		    	time = daisy_get_info (daisy, DAISY_BOOKINFO_TOTALTIME);
		    	if (time != NULL)
		    	{
				total_time_box->setText((QString) time);
		    	}
		    	else total_time_box->setText (tr ("Unkown"));
		    	/* Puts the playback buttons correctly. */	
			bookmarkAction->setEnabled (true);
			loadBookmarkAction->setEnabled (false);
			nextChapterAction->setEnabled (true);
		    	nextPassageAction->setEnabled (true);
		    	playAction->setEnabled (true);
		    	pauseAction->setEnabled (false);
		    	prevChapterAction->setEnabled (true);
		    	prevPassageAction->setEnabled (true);
		    	stopAction->setEnabled (false);
		    	/* Fill the chapter navigator. */
	    		fillListBookNav ();
	    	}
	    	else statusBar()->message (tr ("Unable to load book"), 5000);
	}	    
        else statusBar()->message (tr ("File Open abandoned"), 2000);
}

/**
 * Terminates daisy and exits the program.
 */
void MainWindow::fileExit ()
{
    	daisy_term (daisy);
    	QApplication::exit (0);
}

/**
 * Opens the About-dialog.
 */
void MainWindow::helpAbout ()
{
    	AboutDialog *ad = new AboutDialog (this);
    	ad->exec ();
}

/**
 * Starts playing the book.
 */
void MainWindow::menuPlay ()
{	
	int ret = 0;
	
     	ret = daisy_play (daisy);
     	if (ret == 1)
     	{
		/* Puts the playback buttons correctly. */
	    	bookmarkAction->setEnabled (true);
	    	total_time_box->setReadOnly (true);		
		nextPassageAction->setEnabled (true);
    		pauseAction->setEnabled (true);
   		playAction->setEnabled (false);
    		prevChapterAction->setEnabled (true);
	    	prevPassageAction->setEnabled (true);
    		stopAction->setEnabled (true);
    	}
}

/**
 * Loads the previous passage.
 */
void MainWindow::menuPrevPassage ()
{
    	int ret = 0;
    	
    	ret = daisy_seek (daisy, DAISY_SEEK_PREV_PASSAGE);
    	if (ret == 1)
    	{
		/* Puts the playback buttons correctly. */
		bookmarkAction->setEnabled (true);
    		nextChapterAction->setEnabled (true);
    		nextPassageAction->setEnabled (true);
    		pauseAction->setEnabled (true);
    		playAction->setEnabled (false);
    		prevChapterAction->setEnabled (true);
    		prevPassageAction->setEnabled (true);
    		stopAction->setEnabled (true);
    	}
}

/**
 * Loads the previous chapter.
 */
void MainWindow::menuPrevChapter ()
{
    	int ret = 0;
    	
    	ret = daisy_seek(daisy, DAISY_SEEK_PREV_CHAPTER);
	if (ret == 1)
	{
		/* Puts the playback buttons correctly. */
		bookmarkAction->setEnabled (true);
    		nextChapterAction->setEnabled (true);
    		nextPassageAction->setEnabled (true);
		pauseAction->setEnabled (true);
	    	playAction->setEnabled (false);
    		prevChapterAction->setEnabled (true);
    		prevPassageAction->setEnabled (true);
    		stopAction->setEnabled (true);
    	}
}

/**
 * Loads the next chapter.
 */
void MainWindow::menuNextChapter ()
{
    	int ret = 0;
    	
    	ret = daisy_seek(daisy, DAISY_SEEK_NEXT_CHAPTER);
	if (ret == 1)
	{
		/* Puts the playback buttons correctly. */
		bookmarkAction->setEnabled (true);
    		nextChapterAction->setEnabled (true);
    		nextPassageAction->setEnabled (true);
    		pauseAction->setEnabled (true);
    		playAction->setEnabled (false);
    		prevChapterAction->setEnabled (true);
    		prevPassageAction->setEnabled (true);
    		stopAction->setEnabled (true);
    	}
    	/* we are on the last chapter */
    	else if (ret == 0)
    	{
    		statusBar()->message (tr ("You have reached end of book"), 3000);
		daisy_stop (daisy);
    		/* Puts the playback buttons correctly. */
		bookmarkAction->setEnabled (false);
		nextChapterAction->setEnabled (false);
     		nextPassageAction->setEnabled (false);
     		pauseAction->setEnabled (false);
     		playAction->setEnabled (true);
     		prevChapterAction->setEnabled (true);
     		prevPassageAction->setEnabled (true);
     		stopAction->setEnabled (false);
    	}
}

/**
 * Loads the next passage.
 */
void MainWindow::menuNextPassage ()
{
    	int ret = 0;
    	
    	ret = daisy_seek(daisy, DAISY_SEEK_NEXT_PASSAGE);
	if (ret == 1)
	{
		/* Puts the playback buttons correctly. */
		bookmarkAction->setEnabled (true);
    		nextChapterAction->setEnabled (true);
    		nextPassageAction->setEnabled (true);
    		pauseAction->setEnabled (true);
    		playAction->setEnabled (false);
    		prevChapterAction->setEnabled (true);
    		prevPassageAction->setEnabled (true);
    		stopAction->setEnabled (true);
    	}
	/* we are on the last passage, stops the playback */
    	else if (ret == 0)
    	{	
    		statusBar()->message (tr ("You have reached end of book"), 3000);
    		daisy_stop (daisy);
		/* Puts the playback buttons correctly. */
		bookmarkAction->setEnabled (false);
  	     	nextChapterAction->setEnabled (false);
     		nextPassageAction->setEnabled (false);
     		pauseAction->setEnabled (false);
     		playAction->setEnabled (true);
     		prevChapterAction->setEnabled (true);
     		prevPassageAction->setEnabled (true);
     		stopAction->setEnabled (false);
    	}
}

/**
 * Stops playback.
 */
void MainWindow::menuStop ()
{
     	int ret = 0;
     	
     	ret = daisy_stop (daisy);
     	if (ret == 1)
     	{
		/* Puts the playback buttons correctly. */
		bookmarkAction->setEnabled (false);
	     	nextChapterAction->setEnabled (true);
     		nextPassageAction->setEnabled (true);
     		pauseAction->setEnabled (false);
     		playAction->setEnabled (true);
     		prevChapterAction->setEnabled (true);
     		prevPassageAction->setEnabled (true);
     		stopAction->setEnabled (false);
     	}
}

/**
 * Pauses the playback.
 */
void MainWindow::menuPause ()
{
     	int ret = 0;
     	
     	ret = daisy_pause (daisy);
     	if (ret == 1)
     	{
		/* Puts the playback buttons correctly. */
		bookmarkAction->setEnabled (true);
	     	nextChapterAction->setEnabled (true);
     		nextPassageAction->setEnabled (true);
     		pauseAction->setEnabled (true);
     		playAction->setEnabled (false);
     		prevChapterAction->setEnabled (true);
     		prevPassageAction->setEnabled (true);
     		stopAction->setEnabled (true);
     	}
     	else
     		bookmarkAction->setEnabled (true);
}

/**
 * Update the textbrowser with the passage.
 * 
 * @param passage contains the textpassage.
 */
void MainWindow::updateText (const QString &passage )
{
    	emit displayText(passage);
}

/**
 * This function fills a bookmark struct with the current smil/node position.
 */
void MainWindow::menuBookmark()
{	
	/* removes the current bookmark if it exist. */
	if (bookmark != NULL)
	{
		free (bookmark);
		bookmark = NULL;
	}
    	/* allocates a new bookmark struct */
	bookmark = new (daisy_position [sizeof (daisy_position)]);
	/* get the current position */
	bookmark = daisy_get_position(daisy);
	loadBookmarkAction->setEnabled (true);
	statusBar()->message (tr ("Passage succesfully bookmarked"), 3000);
}

/**
 * This function will load a bookmark and go to that position.
 */
void MainWindow::menuLoadBookmark()
{	
    	/* doublecheckes wheter bookmark isn't NULL */
	if (bookmark != NULL)
    	{
		daisy_goto_position (daisy, bookmark); /* jump to bookmarked position */
		statusBar()->message (tr ("Passage succesfully bookmarked"), 3000);
		/* Puts the playback buttons correctly. */
		bookmarkAction->setEnabled (true);
		loadBookmarkAction->setEnabled (true);
	    	total_time_box->setReadOnly (true);		
		nextPassageAction->setEnabled (true);
    		pauseAction->setEnabled (true);
   		playAction->setEnabled (false);
    		prevChapterAction->setEnabled (true);
	    	prevPassageAction->setEnabled (true);
    		stopAction->setEnabled (true);
    	}
    	else
    	{	
   		/* Puts the playback buttons correctly. */
	    	bookmarkAction->setEnabled (true);
		loadBookmarkAction->setEnabled (false);
	    	total_time_box->setReadOnly (true);		
		nextPassageAction->setEnabled (true);
    		pauseAction->setEnabled (true);
   		playAction->setEnabled (false);
    		prevChapterAction->setEnabled (true);
	    	prevPassageAction->setEnabled (true);
    		stopAction->setEnabled (true);
		statusBar()->message (tr ("No bookmark available"), 3000);
    	}    		
}

/**
 * Changes the language to norwegian.
 */
void MainWindow::helpLanguageNorwegian ()
{
	char *title, buffer[300];
	if (daisy != NULL)
	{	
		qt.load (QString ("/usr/share/doc/daisygui/translations/daisygui_no_nb.qm"));
		title = daisy_get_info (daisy, DAISY_BOOKINFO_TITLETEXT);
		if(title != NULL)
		{	
	  		snprintf(buffer, sizeof (buffer), "DaisyPlayer - GUI edition -- %s", title);
	  		setCaption(buffer);
	  		printf("\n\nENOR:%s\n", buffer);
	    	}
	 }
	 else qt.load (QString ("/usr/share/doc/daisygui/translations/daisygui_no_nb.qm"));
}

/**
 * Changes the language to english.
 */
void  MainWindow::helpLanguageEnglish ()
{
	char *title, buffer[300];
	if (daisy != NULL)
	{	
		qt.load (QString ("/usr/share/doc/daisygui/translations/daisygui_en.qm"));
		title = daisy_get_info (daisy, DAISY_BOOKINFO_TITLETEXT);
		if(title != NULL)
		{	
	  		snprintf(buffer, sizeof (buffer), "DaisyPlayer - GUI edition -- %s", title);
	  		setCaption(buffer);
	  		printf("\n\nENG:%s\n", buffer);
	    	}
	 }
	 else qt.load (QString ("/usr/share/doc/daisygui/translations/daisygui_en.qm"));
}
 

/**
  * Method for changing fonts. Is invoked when the user changes the fontsize spinbox.
  * 
  * @param value contains the current value in the spinbox.
  */
void MainWindow::fontsize_spinbox_valueChanged (int value)
{
    	const QFont f (font_combobox->currentText (), value, QFont::Normal);
     	textBrowser->setFont (f);
}

/**
  * Method for changing fonts. Is invoked when the user interacts with the font combobox.
  */
void MainWindow::font_combobox_valueChanged ()
{
    	const QFont f( font_combobox->currentText (), fontsize_spinbox->value (), QFont::Normal);
     	textBrowser->setFont(f);
}

/**
 * Fills listBookNav with chapter info
 * NOTE: Here, we disable sorting and read in the chapters in sequence. The GUI does not sort 
 * the chapters. QListViewItem::compare is overloaded though (the chapterlistviewitem.h|c 
 * files) so a decent sort method should be implementable in the future.
 */
void MainWindow::fillListBookNav ()
{
	ChapterListViewItem *entry = NULL;
	int i = 0;
	
	listBookNav->clear ();    // Clear in case we've already loaded a book.
	listBookNav->setSorting ( -1, TRUE );	// Disable sorting - chapters will populate the list in the order that they are added. (actually sorting the list is probably a more failsafe method.
	
	for (i = daisy_get_chapter_count ( daisy ); i >0 ; i--) {
		entry = new ChapterListViewItem ( listBookNav, QString::number (i), QString::fromUtf8 (daisy_get_chapter_info (daisy, i, DAISY_CHAPTER_TITLE)) );
	}
}

/**
 * Jumps to the right chapter after doubleclicking in the list.
 * @param ChapterListViewItem *si - pointer to the double-clicked item.
 */
void MainWindow::listBookNav_doubleClicked ( QListViewItem *si )
{
	int ret = 0;
	daisy_position jumpTo = {0, 0};
	ChapterListViewItem *selectedItem = (ChapterListViewItem *) si;
	ChapterListViewItem *seekItem = (ChapterListViewItem *) listBookNav->firstChild ();

	while ( (seekItem != NULL) && (seekItem != selectedItem)) 
	{
		seekItem = (ChapterListViewItem *) seekItem->nextSibling ();
	}
	if (seekItem == NULL)
	{
	}
	else if (seekItem == selectedItem) 
	{
		jumpTo.smilpos = seekItem->text (0).toInt ();
		jumpTo.nodepos = 1;
		ret = daisy_goto_position (daisy, &jumpTo);
		if (ret == 1)
		{
		    	/* Puts the playback buttons correctly. */
			nextChapterAction->setEnabled (true);
			nextPassageAction->setEnabled (true);
			pauseAction->setEnabled (true);
			playAction->setEnabled (false);
			prevChapterAction->setEnabled (true);
			prevPassageAction->setEnabled (true);
			stopAction->setEnabled (true);
		}	
	}
}

/*==================*
 *callback functions
 *==================*/
 
/**
 * The callback functon called everytime the daisy engine is done playing a audio segment.
 * @param void *self - pointer to the object that uses libdaisy
 */
void cb_audio_done (void *self)
{
 	self = self;   
}

/**
 * This callback function is called everytime the daisy engine starts to play a new audio segment.
 * @param void *self - pointer to the object that uses libdaisy
 * @param length - a unsigned long int containing the length of the audio segment in ms.
 */
void cb_audio_next (void *self, unsigned long int length)
{
	self = self; length = length;
}

/**
 * This callback function is called everytime the daisy engine starts to play a new audio segment, supplying the corresponding text.
 * @param void *self - pointer to the object that uses libdaisy
 * @param passage - an char* containing the text passage.
 */
void cb_text (void *self, void *_passage)
{	
    MainWindow *mw = (MainWindow*) self;	
    char *passage = (char *)_passage;   
    mw->updateText(QString(passage));
}

/**
 * This callback function is called everytime the daisy enfine start to play a new audio segment, supplying the corresponding id to the text in the xml file.
 * @param void *self - pointer to the object that uses libdaisy
 * @param id - an char* containing the id of the text passage in the xml file.
 */
void cb_id (void *self, void *_id)
{
	self = self; _id = _id;
}

/**
 * This callback function is called everytime an error occurs in the daisy engine.
 * @param void *self - pointer to the object that uses libdaisy
 * @param err - an enum daisy_status containing the error code of error type.
 * @param daisy_error_msg - an char* containing the error message. This can be NULL when an message is not needed to explain the error.
 */
void cb_error (void *self, enum daisy_status err, const char *daisy_error_msg)
{
    self = self; err = err; daisy_error_msg = daisy_error_msg;
    /*    printf ("Error: (%d) - (%s)\n", err, daisy_error_msg);*/
}

/**
 * This callback function is called during playback, supplying the progress in ms.
 * @param void *self - pointer to the object that uses libdaisy
 * @param progress_ms - an long int containing the progress in ms.
 */
void cb_progress (void *self, long int progress_ms)
{	
	self = self; progress_ms = progress_ms;
}
